/* 
 * ctype.h
 *
 * Functions for testing character types and converting characters.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _CTYPE_H_
#define _CTYPE_H_



#ifdef __cplusplus
extern "C" {
#endif

int _cdecl	isalnum(int c);
int _cdecl	isalpha(int c);
int _cdecl	iscntrl(int c);
int _cdecl	isdigit(int c);
int _cdecl	isgraph(int c);
int _cdecl	islower(int c);
int _cdecl	isprint(int c);
int _cdecl 	ispunct(int c);
int _cdecl	isspace(int c);
int _cdecl	isupper(int c);
int _cdecl	isxdigit(int c);
int _cdecl	tolower(int c);
int _cdecl	toupper(int c);

extern unsigned short *_imp___ctype[];
#define _ctype_		(*_imp___ctype)

/* The following flags are used to tell iswctype and _isctype what character
 * types you are looking for. */
#define _U      0x01
#define _L      0x02
#define _N      0x04
#define _S      0x08
#define _P      0x10
#define _C      0x20
#define _B      0x40
#define _X      0x80

#ifndef _NO_CTYPE_MACROS_
#ifdef __STRICT_ANSI__
#define isalpha(c)      ((_ctype_+1)[c]&(_U|_L))
#define isupper(c)      ((_ctype_+1)[c]&_U)
#define islower(c)      ((_ctype_+1)[c]&_L)
#define isdigit(c)      ((_ctype_+1)[c]&_N)
#define isxdigit(c)     ((_ctype_+1)[c]&(_X|_N))
#define isspace(c)      ((_ctype_+1)[c]&_S)
#define ispunct(c)      ((_ctype_+1)[c]&_P)
#define isalnum(c)      ((_ctype_+1)[c]&(_U|_L|_N))
#define isprint(c)      ((_ctype_+1)[c]&(_P|_U|_L|_N|_B))
#define isgraph(c)      ((_ctype_+1)[c]&(_P|_U|_L|_N))
#define iscntrl(c)      ((_ctype_+1)[c]&_C)
/* Non-gcc versions will get the library versions, and will be
   slightly slower */
#ifdef __GNUC__
# define toupper(c) \
        ({ int __x = (c); islower(__x) ? (__x - 'a' + 'A') : __x;})
# define tolower(c) \
        ({ int __x = (c); isupper(__x) ? (__x - 'A' + 'a') : __x;})
#endif // __GNUC__
#else // !__STRICT_ANSI__

extern unsigned short** _cdecl __p__pctype(void);
#define _pctype (*__p__pctype())

#ifndef MB_CUR_MAX
int* _cdecl __p___mb_cur_max(void);
#define MB_CUR_MAX (*__p___mb_cur_max())
#endif //MB_CUR_MAX

#define isalpha(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&(_U|_L) : (_pctype)[c]&(_U|_L) )
#define isupper(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_U : (_pctype)[c]&_U )
#define islower(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_L : (_pctype)[c]&_L )
#define isdigit(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_N : (_pctype)[c]&_N )
#define isxdigit(c)     (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&(_X|_N) : (_pctype)[c]&(_X|_N) )
#define isspace(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_S : (_pctype)[c]&_S )
#define ispunct(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_P : (_pctype)[c]&_P )
#define isalnum(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&(_U|_L|_N) : (_pctype)[c]&(_U|_L|_N) )
#define isprint(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&(_P|_U|_L|_N|_B) : (_pctype)[c]&(_P|_U|_L|_N|_B) )
#define isgraph(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&(_P|_U|_L|_N) : (_pctype)[c]&(_P|_U|_L|_N) )
#define iscntrl(c)      (MB_CUR_MAX > 1 ? (_ctype_+1)[c]&_C : (_pctype)[c]&_C )
/* Non-gcc versions will get the library versions, and will be
   slightly slower */
#ifdef __GNUC__
# define toupper(c) \
        ({ int __x = (c); islower(__x) ? (__x - 'a' + 'A') : __x;})
# define tolower(c) \
        ({ int __x = (c); isupper(__x) ? (__x - 'A' + 'a') : __x;})
#endif // __GNUC__


#endif // __STRICT_ANSI

#endif //_NO_CTYPE_MACROS_

#ifndef	__STRICT_ANSI__

#define	_UPPER		_U	
#define	_LOWER		_L
#define	_DIGIT		_N
#define	_SPACE		_S
#define	_PUNCT		_P
#define	_CONTROL	_C
#define	_BLANK		_B
#define	_HEX		_X
#define	_LEADBYTE	0x8000

#define	_ALPHA		0x0103

#ifndef WEOF
#define	WEOF	(wchar_t)(0xFFFF)
#endif

#define _ctype _ctype_

#define	__need_wchar_t
#define	__need_wint_t
#include <stddef.h>

typedef wchar_t	wctype_t;

extern wctype_t** _cdecl __p__pwctype(void);
#define _pwcytpe (*__p__pwctype())

/* Wide character equivalents */
int _cdecl	iswalnum(wint_t wc);
int _cdecl	iswalpha(wint_t wc);
int _cdecl	iswascii(wint_t wc);
int _cdecl	iswcntrl(wint_t wc);
int _cdecl	iswctype(wint_t wc, wctype_t wctypeFlags);
int _cdecl	is_wctype(wint_t wc, wctype_t wctypeFlags);	/* Obsolete! */
int _cdecl	iswdigit(wint_t wc);
int _cdecl	iswgraph(wint_t wc);
int _cdecl	iswlower(wint_t wc);
int _cdecl	iswprint(wint_t wc);
int _cdecl	iswpunct(wint_t wc);
int _cdecl	iswspace(wint_t wc);
int _cdecl	iswupper(wint_t wc);
int _cdecl	iswxdigit(wint_t wc);

wchar_t	_cdecl towlower(wchar_t c);
wchar_t	_cdecl towupper(wchar_t c);

int _cdecl	isleadbyte (int c);


int _cdecl	_isctype (int c, int ctypeFlags);
int _cdecl	_tolower(int c);
int _cdecl	_toupper(int c);
int _cdecl	__isascii (int c);
int _cdecl	__toascii (int c);
int _cdecl	__iscsymf (int c);	/* Valid first character in C symbol */
int _cdecl	__iscsym (int c);	/* Valid character in C symbol (after first) */
int _cdecl	isascii (int c);
int _cdecl	toascii (int c);
int _cdecl	iscsymf (int c);
int _cdecl	iscsym (int c);

#ifndef _NO_CTYPE_MACROS_
#define _tolower tolower
#define _toupper toupper
#define __isascii(c)      ((unsigned)(c) < 0x80)
#define __toascii(c)      ((c) & 0x7f)
#define __iscsymf(c)	(isalpha(c) || ((c) == '_'))
#define __iscsym(c)	(isalnum(c) || ((c) == '_'))
#define isascii __isascii
#define toascii __toascii
#define iscsymf __iscsymf
#define iscsym __iscsym
#endif //_NO_CTYPE_MACROS_

#endif	/* Not __STRICT_ANSI__ */

#ifdef __cplusplus
}
#endif

#endif //_CTYPE_H_
