/* 
 * errno.h
 *
 * Error numbers and access to error reporting.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _ERRNO_H_
#define	_ERRNO_H_

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Error numbers.
 * TODO: Can't be sure of some of these assignments, I guessed from the
 * names given by strerror and the defines in the Cygnus errno.h. A lot
 * of the names from the Cygnus errno.h are not represented, and a few
 * of the descriptions returned by strerror do not obviously match
 * their error naming.
 */
#define	EDOM		33	/* win32 Math argument (Domain error) (math functions) */
#define	ERANGE		34	/* win32 Result too large(possibly too small) An argument to a math function is too large, resulting in partial or total loss of significance in the result. This error can also occur in other functions when an argument is larger than expected (for example, when the buffer argument to _getcwd is longer than expected). */

#ifndef errno
int*    _errno();
#define errno           (*_errno())
#endif //errno

#ifndef __STRICT_ANSI__

#define EPERM		1	/* Operation not permitted */
#define	ENOFILE		2	/* win32 No such file or directory */
#define	ENOENT		2	/*The specified file or directory does not exist or cannot be found. This message can occur whenever a specified file does not exist or a component of a path does not specify an existing directory. */
#define	ESRCH		3	/* No such process */
#define	EINTR		4	/* Interrupted function call */
#define	EIO		5	/* Input/output error */
#define	ENXIO		6	/* No such device or address */
#define	E2BIG		7	/* win32 Arg list too long */
#define	ENOEXEC		8	/* win32 Exec format error An attempt was made to execute a file that is not executable or that has an invalid executable-file format */
#define	EBADF		9	/* win32 Bad file descriptor There are two possible causes: 1) The specified file handle is not a valid file-handle value or does not refer to an open file. 2) An attempt was made to write to a file or device opened for read-only access */
#define	ECHILD		10	/* win32 No child processes */
#define	EAGAIN		11	/* win32 Resource temporarily unavailable An attempt to create a new process failed because there are no more process slots, or there is not enough memory, or the maximum nesting level has been reached. */
#define	ENOMEM		12	/* win32 Not enough space Not enough core. Not enough memory is available for the attempted operator. For example, this message can occur when insufficient memory is available to execute a child process, or when the allocation request in a _getcwd call cannot be satisfied */
#define	EACCES		13	/* win32 Permission denied The file's permission setting does not allow the specified access. This error signifies that an attempt was made to access a file (or, in some cases, a directory) in a way that is incompatible with the file's attributes. For example, the error can occur when an attempt is made to read from a file that is not open, to open an existing read-only file for writing, or to open a directory instead of a file. Under MS-DOS operating system versions 3.0 and later, EACCES may also indicate a locking or sharing violation.

The error can also occur in an attempt to rename a file or directory or to remove an existing directory. */
#define	EFAULT		14	/* Bad address */
/* 15 - Unknown Error */
#define	EBUSY		16	/* strerror reports "Resource device" */
#define	EEXIST		17	/* win32 File exists An attempt has been made to create a file that already exists. For example, the _O_CREAT and _O_EXCL flags are specified in an _open call, but the named file already exists. */
#define	EXDEV		18	/* win32 Improper link (cross-device link) Cross-device link. An attempt was made to move a file to a different device (using the rename function). */
#define	ENODEV		19	/* No such device */
#define	ENOTDIR		20	/* Not a directory */
#define	EISDIR		21	/* Is a directory */
#define	EINVAL		22	/* win32 Invalid argument An invalid value was given for one of the arguments to a function. For example, the value given for the origin when positioning a file pointer (by means of a call to fseek) is before the beginning of the file */
#define	ENFILE		23	/* Too many open files in system */
#define	EMFILE		24	/* win32 Too many open files. No more file handles are available, so no more files can be opened */
#define	ENOTTY		25	/* Inappropriate I/O control operation */
/* 26 - Unknown Error */
#define	EFBIG		27	/* File too large */
#define	ENOSPC		28	/* win32 No space left on device No space left on device. No more space for writing is available on the device (for example, when the disk is full). */
#define	ESPIPE		29	/* Invalid seek (seek on a pipe?) */
#define	EROFS		30	/* Read-only file system */
#define	EMLINK		31	/* Too many links */
#define	EPIPE		32	/* Broken pipe */
/* 35 - Unknown Error */
#define	EDEADLOCK	36	/* win32 Resource deadlock avoided (non-Cyg) Resource deadlock would occur. The argument to a math function is not in the domain of the function */
#define	EDEADLK		36
/* 37 - Unknown Error */
#define	ENAMETOOLONG	38	/* Filename too long (91 in Cyg?) */
#define	ENOLCK		39	/* No locks available (46 in Cyg?) */
#define	ENOSYS		40	/* Function not implemented (88 in Cyg?) */
#define	ENOTEMPTY	41	/* Directory not empty (90 in Cyg?) */
#define	EILSEQ		42	/* Illegal byte sequence */

#endif //__STRICT_ANSI__

#ifdef	__cplusplus
}
#endif

#endif
