/* 
 * io.h
 *
 * System level I/O functions and types.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef	_IO_H_
#define	_IO_H_

#ifndef	_STRICT_ANSI

/* We need the definition of FILE anyway... */
#include <stdio.h>

/* MSVC's io.h contains the stuff from dir.h, so I will too. */
#include <dir.h> // mkdir rmdir mktemp getcwd
#include <stdlib.h>
#include <sys/types.h>
#define __need___int64
#include <stddef.h>

/* TODO: Maximum number of open handles has not been tested, I just set
 * it the same as FOPEN_MAX. */
#define	HANDLE_MAX	FOPEN_MAX

#ifndef _FSIZE_T_DEFINED
typedef unsigned long   _fsize_t;
#define _FSIZE_T_DEFINED
#endif

/*
 * The following structure is filled in by _findfirst or _findnext when
 * they succeed in finding a match.
 */
struct _finddata_t
{
        unsigned        attrib;         /* Attributes, see constants above. */
        time_t          time_create;
        time_t          time_access;    /* always midnight local time */
        time_t          time_write;
        _fsize_t        size;
        char            name[_MAX_FNAME];       /* may include spaces. */
};

struct _wfinddata_t
{
        unsigned        attrib;
        time_t          time_create;
        time_t          time_access;
        time_t          time_write;
        _fsize_t        size;
        wchar_t         name[_MAX_FNAME];
};

struct _wfinddatai64_t
{
        unsigned        attrib; //Attributes, see constants above.
        time_t          time_create;
        time_t          time_access;    // always midnight local time
        time_t          time_write;
        __int64        size;
        char            name[_MAX_FNAME];       // may include spaces
};

/* Attributes of files as returned by _findfirst et al.
 */
#define _A_NORMAL       0x00000000
#define _A_RDONLY       0x00000001
#define _A_HIDDEN       0x00000002
#define _A_SYSTEM       0x00000004
#define _A_VOLID        0x00000008
#define _A_SUBDIR       0x00000010
#define _A_ARCH         0x00000020



/* Some defines for _access nAccessMode (MS doesn't define them, but
 * it doesn't seem to hurt to add them). */
#define	F_OK	0	/* Check for file existence */
#define	W_OK	2	/* Check for write permission */
#define	R_OK	4	/* Check for read permission */
/* TODO: Is this safe? X_OK not supported directly... */
#define X_OK	R_OK	/* Check for execute permission */

#ifdef	__cplusplus
extern "C" {
#endif

#if defined(_WIN32_WINNT)

int _cdecl _waccess( const wchar_t *path, int mode );
int _cdecl _wchmod( const wchar_t *filename, int pmode );
int _cdecl _wcreat( const wchar_t *filename, int pmode );
int _cdecl _wrename( const wchar_t *oldname, const wchar_t *newname );
int _cdecl _wunlink( const wchar_t *filename );
long _cdecl _wfindfirst( wchar_t *filespec, struct _wfinddata_t *fileinfo );
__int64 _cdecl _wfindfirsti64( wchar_t *filespec, struct _wfinddata_t *fileinfo );
int _cdecl _wfindnext( long handle, struct _wfinddata_t *fileinfo );
__int64 _cdecl _wfindnexti64( long handle, struct _wfinddata_t *fileinfo );
int _cdecl _wopen( const wchar_t *filename, int oflag , int pmode );
int _cdecl _wsopen( const wchar_t *filename, int oflag, int shflag , int pmode );

#endif //_WIN32_WINNT

int _cdecl		_access (const char* szFileName, int nAccessMode);
int _cdecl		_chsize (int nHandle, long lnNewSize);
int _cdecl             _chmod( const char *filename, int pmode );
int _cdecl		_close (int nHandle);
int _cdecl 		_commit( int handle );
int _cdecl		_creat (const char* szFileName, int nAccessMode);
int _cdecl		_dup (int nHandle);
int _cdecl		_dup2 (int nOldHandle, int nNewHandle);
FILE*		_fdopen (int nHandle, const char* szMode);
long _cdecl		_filelength (int nHandle);
int _cdecl		_fileno (FILE* fileGetHandle);
long _cdecl		_get_osfhandle (int nHandle);
int _cdecl		_isatty (int nHandle);

/*
 * Functions for searching for files. _findfirst returns -1 if no match
 * is found. Otherwise it returns a handle to be used in _findnext and
 * _findclose calls. _findnext also returns -1 if no match could be found,
 * and 0 if a match was found. Call _findclose when you are finished.
 */

int _cdecl     _findfirst (const char* szFilespec, struct _finddata_t* find);
int _cdecl     _findnext (int nHandle, struct _finddata_t* find);
__int64 _cdecl _findfirsti64( char *filespec, struct _finddata_t *fileinfo );
__int64 _cdecl _findnexti64( long handle, struct _finddata_t *fileinfo );
int _cdecl     _findclose (int nHandle);

__int64 _cdecl _lseeki64( int handle, __int64 offset, int origin );
__int64 _cdecl _telli64( int handle );

/* In a very odd turn of events this function is excluded from those
 * files which define _STREAM_COMPAT. This is required in order to
 * build GNU libio because of a conflict with _eof in streambuf.h
 * line 107. Actually I might just be able to change the name of
 * the enum member in streambuf.h... we'll see. TODO */
#ifndef	_STREAM_COMPAT
int _cdecl		_eof (int nHandle);
#endif

/* LK_... locking commands defined in sys/locking.h. */
int _cdecl		_locking (int nHandle, int nCmd, long lnLockRegionLength);

long _cdecl		_lseek (int nHandle, long lnOffset, int nOrigin);
int _cdecl		_open (const char* szFileName, int nFlags, ...);
int _cdecl		_open_osfhandle (long lnOSHandle, int nFlags);
int _cdecl		_pipe (int *naHandles, unsigned int unSize, int nMode);
int _cdecl		_read (int nHandle, void* caBuffer, unsigned int nToRead);
int _cdecl 		remove( const char *path );
int _cdecl 		rename( const char *oldname, const char *newname );
/* SH_... flags for nFlag defined in share.h */
int _cdecl     	_setmode (int nHandle, int nAccessMode);
int _cdecl		_sopen (char* szFileName, int nAccess, int nFlag, int nMode);

long _cdecl		_tell (int nHandle);
unsigned int _cdecl	_umask (unsigned int unMode);
int _cdecl		_unlink (const char* szFileName);
int _cdecl		_write (int nHandle, const void* caBuffer,
		        unsigned int nToWrite);


#ifndef	_NO_OLDNAMES
/*
 * Non-underscored versions of non-ANSI functions to improve portability.
 * These functions live in libmoldname.a.
 */

int _cdecl		access (const char* szFileName, int nAccessMode);
int _cdecl		chsize (int nHandle, long lnNewSize);
int _cdecl 		chmod( const char *filename, int pmode );
int _cdecl		close (int nHandle);
int _cdecl		creat (const char* szFileName, int nAccessMode);
int _cdecl		dup (int nHandle);
int _cdecl		dup2 (int nOldHandle, int nNewHandle);
int _cdecl		eof (int nHandle);
long _cdecl		filelength (int nHandle);
int _cdecl		fileno (FILE* fileGetHandle);
int _cdecl		isatty (int nHandle);
long _cdecl		lseek (int nHandle, long lnOffset, int nOrigin);
int _cdecl		open (const char* szFileName, int nFlags, ...);
int _cdecl		read (int nHandle, void* caBuffer, unsigned int nToRead);
int _cdecl     	setmode (int nHandle, int nAccessMode);
int _cdecl		sopen (char* szFileName, int nAccess, int nFlag, int nMode);
long _cdecl		tell (int nHandle);
unsigned int _cdecl	umask (unsigned int unMode);
int _cdecl		unlink (const char* szFileName);
int _cdecl		write (int nHandle, const void* caBuffer,
		       unsigned int nToWrite);

#endif	/* Not _NO_OLDNAMES */

#ifdef	__cplusplus
}
#endif

#endif	/* Not strict ANSI */

#endif	/* _IO_H_ not defined */
