/* 
 * limits.h
 *
 * Defines constants for the sizes of integral types.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _LIMITS_H_
#define _LIMITS_H_

/*
 * Characteristics of the char data type.
 *
 * TODO: Multibyte char maximum length: MB_LEN_MAX
 */
#define CHAR_BIT 8

#define SCHAR_MIN	(-128)
#define SCHAR_MAX	127

#define UCHAR_MAX	0xFF

/* /J option == gcc -funsigned-char default -fsigned-char for MS & gcc */
#define CHAR_MIN ( (2 * ((char)128)) < 256 ? (-128) : (0))
#define CHAR_MAX ( (2 * ((char)128)) < 256 ? (127) : (256))

/*
 * Maximum and minimum values for ints.
 */
#define INT_MAX		2147483647
#define INT_MIN		(-INT_MAX-1)

#define UINT_MAX	(2U * INT_MAX + 1)

/*
 * Maximum and minimum values for shorts.
 */
#define SHRT_MAX	32767
#define SHRT_MIN	(-SHRT_MAX-1)

#define USHRT_MAX	((unsigned short) (2U * SHRT_MAX + 1))

/*
 * Maximum and minimum values for longs and unsigned longs.
 *
 * TODO: This is not correct for Alphas, which have 64 bit longs.
 */
#define LONG_MAX	2147483647L

#define LONG_MIN	(-LONG_MAX-1)

#define ULONG_MAX	(2UL * LONG_MAX + 1)

#ifndef __STRICT_ANSI__

#define LONG_LONG_MAX	9223372036854775807LL
#define LONG_LONG_MIN	(-LONG_LONG_MAX-1)

#define ULONG_LONG_MAX	(2ULL * LONG_LONG_MAX + 1)

#define MB_LEN_MAX 2

#define ARG_MAX             14500
#define LINK_MAX            1024
#define MAX_CANON           255
#define MAX_INPUT           255
#define NAME_MAX            255
#define NGROUPS_MAX         16
#define OPEN_MAX            32
#define PATH_MAX            512
#define PIPE_BUF            512
#define SSIZE_MAX           32767
#ifndef STREAM_MAX
#define STREAM_MAX          20
#endif
#define TZNAME_MAX          10

#define _POSIX_ARG_MAX      4096
#define _POSIX_CHILD_MAX    6
#define _POSIX_LINK_MAX     8
#define _POSIX_MAX_CANON    MAX_CANNON
#define _POSIX_MAX_INPUT    MAX_INPUT
#define _POSIX_NAME_MAX     14
#define _POSIX_NGROUPS_MAX  0
#define _POSIX_OPEN_MAX     16
#define _POSIX_PATH_MAX     255
#define _POSIX_PIPE_BUF     PIPE_BUF
#define _POSIX_SSIZE_MAX    SSIZE_MAX
#define _POSIX_STREAM_MAX   8
#define _POSIX_TZNAME_MAX   3

#endif //__STRICT_ANSI__


#endif /* not _LIMITS_H_ */
