/* 
 * process.h
 *
 * Function calls for spawning child processes.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef	_PROCESS_H_
#define	_PROCESS_H_

/*
 * This file is not part of ANSI.
 */
#ifndef	__STRICT_ANSI__

#ifdef	__cplusplus
extern "C" {
#endif

#include <stdlib.h>

/*
 * The functions _beginthreadex and _endthreadex are not provided by CRTDLL.
 * NOTE: Apparently _endthread calls CloseHandle on the handle of the thread,
 * making for race conditions if you are not careful. Basically you have to
 * make sure that no-one is going to do *anything* with the thread handle
 * after the thread calls _endthread or returns from the thread function.
 *
 * NOTE: No old names for these functions. Use the underscore.
 */

typedef void _cdecl (*_threadStart_t)(void *);
unsigned long
	_beginthread	( _threadStart_t pfuncStart,
			 unsigned unStackSize, void* pArgList);

typedef unsigned _stdcall (*_threadStartEx_t)(void *);
unsigned long 
        _beginthreadex( void *security, unsigned stack_size,
			_threadStartEx_t start_address,
			void *arglist, unsigned initflag, unsigned *thrdaddr );

void	_endthread( void );
void    _endthreadex( unsigned retval );

extern unsigned long __threadid(void);
#define _threadid (__threadid())
extern unsigned long __threadhandle(void);

void	_cexit( void );
void	_c_exit( void );

int	_cwait (int* pnStatus, int nPID, int nAction);

int	_getpid();

int	_execl		(const char* szPath, const char* szArgv0, ...);
int	_execle		(const char* szPath, const char* szArgv0, ...);
int	_execlp		(const char* szPath, const char* szArgv0, ...);
int	_execlpe	(const char* szPath, const char* szArgv0, ...);
int	_execv		(const char* szPath, char* const* szaArgv);
int	_execve		(const char* szPath, char* const* szaArgv,
			 char* const* szaEnv);
int	_execvp		(const char* szPath, char* const* szaArgv);
int	_execvpe	(const char* szPath, char* const* szaArgv,
			 char* const* szaEnv);
#ifdef _WIN32_WINNT
int _wexecl( const wchar_t *cmdname, const wchar_t *arg0, ...);
int _wexecle( const wchar_t *cmdname, const wchar_t *arg0, ...);
int _wexeclp( const wchar_t *cmdname, const wchar_t *arg0, ...);
int _wexeclpe( const wchar_t *cmdname, const wchar_t *arg0, ...);
int _wexecv( const wchar_t *cmdname, const wchar_t *const *argv );
int _wexecve( const wchar_t *cmdname, const wchar_t *const *argv,
		 const wchar_t *const *envp );
int _wexecvp( const wchar_t *cmdname, const wchar_t *const *argv );
int _wexecvpe( const wchar_t *cmdname, const wchar_t *const *argv,i
		 const wchar_t *const *envp );
#endif


int	_spawnl		(int nMode, const char* szPath, const char* szArgv0,
			 ...);
int	_spawnle	(int nMode, const char* szPath, const char* szArgv0,
			 ...);
int	_spawnlp	(int nMode, const char* szPath, const char* szArgv0,
			 ...);
int	_spawnlpe	(int nMode, const char* szPath, const char* szArgv0,
			 ...);
int	_spawnv		(int nMode, const char* szPath, char* const* szaArgv);
int	_spawnve	(int nMode, const char* szPath, char* const* szaArgv,
			 char* const* szaEnv);
int	_spawnvp	(int nMode, const char* szPath, char* const* szaArgv);
int	_spawnvpe	(int nMode, const char* szPath, char* const* szaArgv,
			 char* const* szaEnv);

/*
 * Functions without the leading underscore, for portability. These functions
 * live in liboldnames.a.
 * NOTE: execl... and spawnl... are more than simple wrappers, and they
 * actually end up calling execv... or spawnv... (and allocating memory).
 */
int	cwait (int* pnStatus, int nPID, int nAction);
int	getpid ();
int	execl (const char* szPath, const char* szArgv0, ...);
int	execle (const char* szPath, const char* szArgv0, ...);
int	execlp (const char* szPath, const char* szArgv0, ...);
int	execlpe (const char* szPath, const char* szArgv0, ...);
int	execv (const char* szPath, char* const* szaArgv);
int	execve (const char* szPath, char* const* szaArgv, char* const* szaEnv);
int	execvp (const char* szPath, char* const* szaArgv);
int	execvpe (const char* szPath, char* const* szaArgv,
	         char* const* szaEnv);
int	spawnl (int nMode, const char* szPath, const char* szArgv0, ...);
int	spawnle (int nMode, const char* szPath, const char* szArgv0, ...);
int	spawnlp (int nMode, const char* szPath, const char* szArgv0, ...);
int	spawnlpe (int nMode, const char* szPath, const char* szArgv0, ...);
int	spawnv (int nMode, const char* szPath, char* const* szaArgv);
int	spawnve (int nMode, const char* szPath, char* const* szaArgv,
	         char* const* szaEnv);
int	spawnvp (int nMode, const char* szPath, char* const* szaArgv);
int	spawnvpe (int nMode, const char* szPath, char* const* szaArgv,
	          char* const* szaEnv);

int __set_app_type(int); // don't know what else but starts faster w/this

#define _CUI_APP (1)
#define _GUI_APP (2)
#define _NATIVE_APP (0)
#define _REPORT_APP_TYPE (3)


#define _WAIT_CHILD	0 // these are ignored in win32 according to msdn
#define _WAIT_GRANDCHILD	1
#define _P_OVERLAY	2
#define _P_WAIT		0
#define _P_NOWAIT	1
#define _OLD_P_OVERLAY	2
#define _P_NOWAITO	3
#define _P_DETACH	8 // guessed from Create Process

#define WAIT_CHILD	_WAIT_CHILD
#define WAIT_GRANDCHILD	_WAIT_GRANDCHILD
#define P_OVERLAY	_P_OVERLAY
#define P_WAIT		_P_WAIT
#define P_NOWAIT	_P_NOWAIT
#define OLD_P_OVERLAY	_OLD_P_OVERLAY
#define P_NOWAITO	_P_NOWAITO
#define P_DETACH	_P_DETACH


#ifdef	__cplusplus
}
#endif

#endif	/* Not __STRICT_ANSI__ */

#endif	/* _PROCESS_H_ not defined */
