/*
 * stdio.h
 *
 * Definitions of types and prototypes of functions for standard input and
 * output.
 *
 * NOTE: The file manipulation functions provided by CRTDLL.DLL seem to
 * work with either slash (/) or backslash (\) as the path separator.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _STDIO_H_
#define	_STDIO_H_

#ifdef __cplusplus
extern "C" {
#endif

#define __need_size_t
#define __need_NULL
#define __need_wchar_t
#include <stddef.h>
#include <stdarg.h>

/* FILE is actually used as a pointer to an opaque data type.
 * Note: You (as a programmer using the standard library) may only use
 * this type as a pointer because the compiler does not know how much
 * storage to allocate for it. There is no reason I can think of to
 * use it in any other way (unless you were writing the standard
 * library yourself) */

/* The maximum number of files (streams) that may be open at once.
 * acording to vc++ docs 40? streams, or 256 files */
#define FOPEN_MAX	(20)

struct io_buf{
char this_io_buffer[32];
};

typedef struct io_buf FILE;

extern FILE* _imp___iob[FOPEN_MAX];
#define _iob (*_imp___iob)
#define stdin (&_iob[0])
#define stdout (&_iob[1])
#define stderr (&_iob[2])

typedef long long fpos_t; //use %I64d for printf etc.

int	fgetpos	(FILE* , fpos_t* );
int	fsetpos (FILE* , fpos_t* );

// File Operations
FILE*	fopen (const char* szFileName, const char* szMode);
FILE    freopen (const char* szNewFileName, const char* szNewMode,
		 FILE* fileChangeAssociation);
int	fflush (FILE* fileFlush);
int	fclose (FILE* fileClose);
int	remove (const char* szFileName);
int	rename (const char* szOldFileName, const char* szNewFileName);
FILE*	tmpfile ();

char*	tmpnam (char caName[]);

int	setvbuf (FILE* fileSetBuffer, char* caBuffer, int nMode,
		 size_t sizeBuffer);
void _cdecl	setbuf (FILE* fileSetBuffer, char* caBuffer);

// Formatted Output
int	fprintf (FILE* filePrintTo, const char* szFormat, ...);
int	printf (const char* szFormat, ...);
int	sprintf (char* caBuffer, const char* szFormat, ...);
int	vfprintf (FILE* filePrintTo, const char* szFormat, va_list varg);
int	vprintf (const char* szFormat, va_list varg);
int	vsprintf (char* caBuffer, const char* szFormat, va_list varg);

// Formatted Input 
int	fscanf (FILE* fileReadFrom, const char* szFormat, ...);
int	scanf (const char* szFormat, ...);
int	sscanf (const char* szReadFrom, const char* szFormat, ...);

// Character Input and Output Functions
int	fgetc (FILE* fileRead);
char*	fgets (char* caBuffer, int nBufferSize, FILE* fileRead);
int	fputc (int c, FILE* fileWrite);
int	fputs (const char* szOutput, FILE* fileWrite);
int	getc (FILE* fileRead);
int	getchar ();
char*	gets (char* caBuffer);	/* Unsafe: how does gets know how long the
				 * buffer is? */
int	putc (int c, FILE* fileWrite);
int	putchar (int c);
int	puts (const char* szOutput);
int	ungetc (int c, FILE* fileWasRead);

// Direct Input and Output Functions 
size_t	fread (void* pBuffer, size_t sizeObject, size_t sizeObjCount,
		FILE* fileRead);
size_t	fwrite (const void* pObjArray, size_t sizeObject, size_t sizeObjCount,
		FILE* fileWrite);

//File Positioning Constants/Functions
#ifndef SEEK_SET
#define SEEK_SET	(0)
#define	SEEK_CUR	(1)
#define SEEK_END	(2)
#endif //SEEK_SET
int	fseek	(FILE* fileSetPosition, long lnOffset, int nOrigin);
long	ftell	(FILE* fileGetPosition);
void _cdecl	rewind	(FILE* fileRewind);

//Error Functions
void _cdecl	clearerr (FILE* fileClearErrors);
int	feof (FILE* fileIsAtEnd);
int	ferror (FILE* fileIsError);
void	perror (const char* szErrorMessage);

/* The maximum length of a file name. You should use GetVolumeInformation
 * instead of this constant. But hey, this works.
 * NOTE: This is used in the structure _finddata_t (see dir.h) so changing it
 *       is probably not a good idea.  */
#define	FILENAME_MAX	(260)

/* The maximum size of name (including NUL) that will be put in the user
 * supplied buffer caName.
 * NOTE: This has not been determined by experiment, but based on the
 * maximum file name length above it is probably reasonable */
#define	L_tmpnam	FILENAME_MAX


//Returned by various functions on end of file condition or error
#define	EOF	(-1)

// buffering mode (nMode) values for setvbuf
#define _IOFBF          0x0000
#define _IOLBF          0x0040
#define _IONBF          0x0004

/* The buffer size as used by setbuf such that it is equivalent to
 * (void) setvbuf(fileSetBuffer, caBuffer, _IOFBF, BUFSIZ).  */
#define	BUFSIZ	512

#define TMP_MAX		32767

#ifndef __STRICT_ANSI__
#define __need_wint_t
#include <stddef.h>

#ifdef _WIN32_WINNT
int _cdecl		_wremove( const wchar_t *path );
int _cdecl		_wrename( const wchar_t *oldname, const wchar_t *newname );
int _cdecl		_wunlink( const wchar_t *filename );
#define _wP_tmpdir L"\\"
#endif //_WIN32_WINNT

int _cdecl		_fcloseall(void);
FILE* 		_fdopen(int, const char *);
int _cdecl		_fgetchar(void);
int		fgetwc (FILE* fileRead);
wint_t		_fgetwchar(void);
wchar_t*	fgetws (wchar_t* wcaBuffer, int nBufferSize, FILE* fileRead);
int _cdecl		_filbuf(FILE *);
int _cdecl		_fileno(FILE *);
int _cdecl		_flsbuf(int, FILE *);
int _cdecl		_flushall( void );
int _cdecl		_fputchar(int);
int		fputwc (wchar_t wc, FILE* fileWrite);
wint_t  	_fputwchar(wint_t);
int		fputws (const wchar_t* wsOutput, FILE* fileWrite);
FILE*  		_fsopen(const char *, const char *, int);
int		fwprintf (FILE* filePrintTo, const wchar_t* wsFormat, ...);
int		fwscanf (FILE* fileReadFrom, const wchar_t* wsFormat, ...);
int		_getmaxstdio(void);
int		_getw(FILE *);
int		getwc (FILE* fileRead);
int		getwchar ();
wchar_t*	getws (wchar_t* wcaBuffer);
wchar_t*	_getws (wchar_t* wcaBuffer);
int		_pclose(FILE *);
FILE*		_popen(const char *, const char *);
int		_putw(int, FILE *);
int		putwc (wchar_t wc, FILE* fileWrite);
wint_t 		putwchar(wint_t);
int		_putws (const wchar_t* wsOutput);
int		putws (const wchar_t* wsOutput);
int _cdecl		_rmtmp( void );
int		_setmaxstdio(int);
int		_snprintf(char *, size_t, const char *, ...);
int		_snwprintf(wchar_t *, size_t, const wchar_t *, ...);
int		swprintf (wchar_t* wcaBuffer, const wchar_t* wsFormat, ...);
int		swscanf (wchar_t* wsReadFrom, const wchar_t* wsFormat, ...);
char*		_tempnam( char *dir, char *prefix );
int		ungetwc (wchar_t wc, FILE* fileWasRead);
int _cdecl		_unlink( const char *filename );
int		vfwprintf (FILE* filePrintTo, const wchar_t* wsFormat, va_list varg);
int _cdecl		_vsnwprintf( wchar_t *buffer, size_t count, const wchar_t *format, va_list argptr );
int _cdecl		_vsnprintf( char *buffer, size_t count, const char *format, va_list argptr);
int		vswprintf (wchar_t* wcaBuffer, const wchar_t* wsFormat, va_list varg);
int		vwprintf (const wchar_t* wsFormat, va_list varg);
FILE*		_wfdopen(int, const wchar_t *);
FILE*		_wfopen(const wchar_t *, const wchar_t *);
FILE*		_wfreopen(const wchar_t *, const wchar_t *, FILE *);
FILE* 		_wfsopen(const wchar_t *, const wchar_t *, int);
void _cdecl		_wperror(const wchar_t *);
FILE*		_wpopen(const wchar_t *, const wchar_t *);
int		wprintf (const wchar_t* wsFormat, ...);
int		wscanf (const wchar_t* wsFormat, ...);
wchar_t*	_wtempnam(wchar_t *dir, wchar_t *prefix);
wchar_t*	_wtmpnam(wchar_t *string);

#ifndef WEOF
#define WEOF (wint_t)(0xFFFF)
#endif

#define _IOREAD 	0x0001
#define _IOWRT		0x0002
#define _IOMYBUF        0x0008
#define _IOEOF          0x0010
#define _IOERR          0x0020
#define _IOSTRG         0x0040
#define _IORW           0x0080
#define _IOAPPEND       0x0200


#define _SYS_OPEN	FOPEN_MAX
#define _NSTREAM_ 512
#define _NFILE	_NSTREAM_
#define _FPOSOFF(x) ((long) (x))

#define _P_tmpdir "\\"
#define P_tmpdir _P_tmpdir

//posix stuff
FILE* 		fdopen(int, const char *);
int _cdecl		fileno(FILE *);
#define L_ctermid   9
#define STREAM_MAX	_NSTREAM_

#endif //__STRICT_ANSI__

#ifdef __cplusplus
}
#endif

#endif /* _STDIO_H_ */
