/*
 * stdlib.h
 *
 * Definitions for common types, variables, and functions.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _STDLIB_H_
#define _STDLIB_H_

#define __need_size_t
#define __need_wchar_t
#define __need_NULL
#include <stddef.h>

#include <malloc.h> // calloc() free() malloc() realloc()
#include <search.h> // bsearch() qsort()

#ifdef __cplusplus
extern "C" {
#endif

#ifdef	__GNUC__
#define	_ATTRIB_NORETURN	__attribute__ ((noreturn))
#else	/* Not __GNUC__ */
#define	_ATTRIB_NORETURN
#endif	/* __GNUC__ */

/*
 * div_t and ldiv_t are structures used to return the results of div and
 * ldiv.
 */
typedef struct { int quot, rem; } div_t;
typedef struct { long quot, rem; } ldiv_t;

void _cdecl	abort	() _ATTRIB_NORETURN;
int _cdecl	abs	(int n);
typedef void (*_atexit_t)(void);
int _cdecl	atexit	(_atexit_t);
double _cdecl	atof	(const char* szNumber);
int _cdecl	atoi	(const char* szNumber);
long _cdecl	atol	(const char* szNumber);
div_t	_cdecl  div	(int nNumerator, int nDenominator);
void _cdecl	exit	(int nStatus) _ATTRIB_NORETURN;
char*	_cdecl  getenv	(const char* szVarName);
long _cdecl	labs	(long n);
ldiv_t	_cdecl  ldiv	(long lNumerator, long lDenominator);
int _cdecl	mblen	(const char* mbs, size_t sizeString);
size_t _cdecl mbstowcs (wchar_t* wcaDest, const char* mbsConvert, size_t size);
int _cdecl	mbtowc	(wchar_t* wcDest, const char* mbConvert, size_t size);
int _cdecl	rand	();
void _cdecl	srand	(unsigned int nSeed);
double _cdecl	strtod	(const char* szNumber, char** pszAfterNumber);
long _cdecl strtol (const char* szNumber, char** pszAfterNumber, int nBase);
unsigned long _cdecl strtoul (const char* szNumber, char** pszAfterNumber, int nBase);
int _cdecl	system	(const char* szCommand);
size_t _cdecl	wcstombs (char* mbsDest, const wchar_t* wsConvert, size_t size);
int _cdecl	wctomb	(char* mbDest, wchar_t wc);

/* These values may be used as exit status codes. */
#define	EXIT_SUCCESS	0
#define	EXIT_FAILURE	1
// RAND_MAX, maximum value returned by rand. The minimum is zero.
#define	RAND_MAX	0x7FFF

#ifndef MB_CUR_MAX
int* _cdecl __p___mb_cur_max(void);
#define MB_CUR_MAX (*__p___mb_cur_max())
#endif //MB_CUR_MAX

#ifndef	__STRICT_ANSI__
#define __need___int64
#include <stddef.h>

typedef int (*_onexit_t)(void);

/* This seems like a convenient place to declare these variables, which
 * give programs using WinMain (or main for that matter) access to main-ish
 * argc and argv. _environ is a pointer to a table of environment variables.
 * NOTE: Strings in argv and environ are ANSI strings.  */

int* _cdecl __p___argc();
#define __argc	(*__p___argc())
char*** _cdecl __p___argv();
#define __argv	(*__p___argv())

char*** _cdecl __p__environ();
#define _environ	(*__p__environ())
#define environ	_environ

char** _cdecl __p__pgmptr();
#define _pgmptr	(*__p__pgmptr())

wchar_t*** _cdecl __p___wargv();
#define __wargv	(*__p___wargv())

wchar_t*** _cdecl __p__wenviron();
#define _wenviron	(*__p__wenviron())

wchar_t** _cdecl __p__wpgmptr();
#define _wpgmptr	(*__p__wpgmptr())


// error global variables 
#ifndef errno
int* _cdecl _errno();
#define errno           (*_errno())
#endif // errno

#ifndef _doserrno
unsigned long * _cdecl  __doserrno(void);
#define _doserrno   (*__doserrno())
#endif //_doserrno

extern int*     _imp___sys_nerr;
#define _sys_nerr       (*_imp___sys_nerr)
#define sys_nerr _sys_nerr

extern char**	_imp___sys_errlist[];
#define _sys_errlist (*_imp___sys_errlist)
#define sys_errlist _sys_errlist

int* _cdecl __p__fileinfo();
#define _fileinfo (*__p__fileinfo())

int* _cdecl __p__fmode();
#define _fmode (*__p__fmode())

int* _cdecl __p__commode();
#define _commode (*__p__commode())

unsigned int* _cdecl __p__osver();
#define _osver (*__p__osver())

unsigned int* _cdecl __p__winmajor();
#define _winmajor (*__p__winmajor())

unsigned int* _cdecl __p__winminor();
#define _winminor (*__p__winminor())

unsigned int* _cdecl __p__winver();
#define _winver (*__p__winver())

#ifdef _WIN32_WINNT

wchar_t* _cdecl _wfullpath( wchar_t *absPath, const wchar_t *relPath, size_t maxLength );

#endif //_WIN32_WINNT

__int64  _cdecl _atoi64( const char *string );
void	 _cdecl _beep (unsigned int, unsigned int);
char*    _cdecl _ecvt( double value, int count, int *dec, int *sign );
char*    _cdecl ecvt( double value, int count, int *dec, int *sign );
void	 _cdecl _exit	(int nStatus) _ATTRIB_NORETURN;
char* _cdecl _fcvt(double, int, int*, int*);
char* _cdecl fcvt(double, int, int*, int*);
char* _cdecl _fullpath( char *absPath, const char *relPath, size_t maxLength );
char* _cdecl  _gcvt(double, int, char*);
char* _cdecl  gcvt(double, int, char*);
char* _cdecl _itoa(int value, char *string, int radix);
char* _cdecl itoa(int value, char *string, int radix);
wchar_t* _cdecl _itow (int, wchar_t*, int);
unsigned long _cdecl  _lrotl(unsigned long, int);
unsigned long _cdecl  _lrotr(unsigned long, int);
char* _cdecl _ltoa( long value, char *string, int radix );
char* _cdecl ltoa( long value, char *string, int radix );
wchar_t* _cdecl _ltow( long value, wchar_t *string, int radix );
wchar_t* _cdecl ltow( long value, wchar_t *string, int radix );
void _cdecl _makepath( char *path, const char *drive, const char *dir, const char *fname, const char *ext );
size_t _cdecl  _mbstrlen( const char* );
_onexit_t _cdecl _onexit( _onexit_t );
_onexit_t _cdecl onexit( _onexit_t );
int _cdecl _putenv (const char* szNameEqValue);
int _cdecl putenv (const char* szNameEqValue);
unsigned _cdecl _rotl(unsigned int, int);
unsigned _cdecl _rotr(unsigned int, int);
void _cdecl _searchenv (const char* szFileName, const char* szVar, char* szFullPathBuf);
void _cdecl searchenv (const char*, const char*, char*);
void _cdecl _seterrormode (int nMode);
int _cdecl _set_error_mode( int nMode );
void	 _cdecl _sleep (unsigned long ulTime);
void _cdecl _splitpath( const char *path, char *drive, char *dir, char *fname, char *ext );
char* _cdecl _ultoa(unsigned long, char*, int);
char* _cdecl ultoa(unsigned long, char*, int);
wchar_t* _cdecl _ultow (unsigned long, wchar_t*, int);
double _cdecl wcstod(const wchar_t* wsNumber, wchar_t** pwsAfterNumber);
long _cdecl wcstol(const wchar_t* wsNum, wchar_t** pwsAfterNum, int nBase);
unsigned long _cdecl wcstoul (const wchar_t* , wchar_t** , int );
wchar_t* _cdecl _wfullpath(wchar_t*, const wchar_t*, size_t);
wchar_t* _cdecl _wgetenv(const wchar_t*);
void _cdecl _wmakepath( wchar_t *path, const wchar_t *drive, const wchar_t *dir, const wchar_t *fname, const wchar_t *ext );
void   _cdecl _wperror(const wchar_t*);
int    _cdecl _wputenv(const wchar_t*);
void   _cdecl _wsearchenv(const wchar_t*, const wchar_t*, wchar_t*);
void _cdecl _wsplitpath( const wchar_t *path, wchar_t *drive, wchar_t *dir, wchar_t *fname, wchar_t *ext );
int    _cdecl _wsystem(const wchar_t*);
int _cdecl _wtoi(const wchar_t*);
unsigned long _cdecl _wtol(wchar_t *szName);

#ifndef __max
#define __max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef __min
#define __min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#ifndef NOMINMAX
#define NOMINMAX
#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif
#endif  /* NOMINMAX */

//Argumens for _set_error_mode()
#define _OUT_TO_DEFAULT 0 // always msgbox?
#define _OUT_TO_STDERR  1
#define _OUT_TO_MSGBOX  2
#define _REPORT_ERRMODE 3

/* The sum of the fields DRIVE+OTHER should not exceed _MAX_PATH -1 */
#define _MAX_DRIVE	3
#define _MAX_DIR	256
#define _MAX_EXT	256
#define _MAX_FNAME	256
#define _MAX_PATH	260

#endif	/* Not __STRICT_ANSI__ */

/*
 * Undefine the no return attribute used in some function definitions
 */
#undef	_ATTRIB_NORETURN

#ifdef __cplusplus
}
#endif

#endif /* _STDLIB_H_ */
