/*
 * stat.h
 *
 * Symbolic constants for opening and creating files, also stat, fstat and
 * chmod functions.
 *
 * This header file is meant for use with CRTDLL.DLL as included with
 * Windows 95(tm) and Windows NT(tm). In conjunction with other versions
 * of the standard C library things may or may not work so well.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Modified for use with msvcrt.dll by jeffdbREMOVETHIS@goodnet.com
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _STRICT_ANSI

#ifndef _STAT_H_
#define _STAT_H_

#include <sys/types.h>

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Constants for the stat st_mode member.
 */

/* #define	S_IFBLK		0x3000 Block 
   #define _S_IFBLK	S_IFBLK not win32 */

#define	S_IEXEC		0000100
#define	S_IWRITE	0000200
#define	S_IREAD		0000400
#define	S_IFIFO		0010000	/* FIFO */
#define	S_IFCHR		0020000	/* Character */
#define	S_IFDIR		0040000	/* Directory */
#define	S_IFREG		0100000	/* Regular */
#define	S_IFMT		0170000	/* File type mask */

#define _S_IEXEC	S_IEXEC
#define _S_IWRITE	S_IWRITE
#define _S_IREAD	S_IREAD
#define _S_IFIFO	S_IFIFO
#define _S_IFCHR	S_IFCHR
#define _S_IFDIR	S_IFDIR
#define _S_IFREG	S_IFREG
#define _S_IFMT		S_IFMT

#define	S_ISDIR(m)	((m) & S_IFDIR)
#define	S_ISFIFO(m)	((m) & S_IFIFO)
#define	S_ISCHR(m)	((m) & S_IFCHR)
#define	S_ISBLK(m)	((m) & S_IFBLK)
#define	S_ISREG(m)	((m) & S_IFREG)

#define _S_ISDIR	S_ISDIR
#define _S_ISFIFO	S_ISFIFO
#define _S_ISCHR	S_ISCHR
#define _S_ISBLK	S_ISBLK
#define _S_ISREG	S_ISREG

#define	S_IRWXU		(S_IREAD | S_IWRITE | S_IEXEC)
#define	S_IXUSR		S_IEXEC
#define	S_IWUSR		S_IWRITE
#define	S_IRUSR		S_IREAD

#define _S_IRWXU	S_IRWXU
#define _S_IXUSR	S_IXUSR
#define _S_IWUSR	S_IWUSR
#define _S_IRUSR	S_IRUSR

/*
 * The structure manipulated and returned by stat and fstat.
 *
 * NOTE: If called on a directory the values in the time fields are not only
 * invalid, they will cause localtime et. al. to return NULL. And calling
 * asctime with a NULL pointer causes an Invalid Page Fault. So watch it!
 */
struct _stat
{
	_dev_t	st_dev;		/* device handle only con aux... otherwise 0 */
	_ino_t	st_ino;		/* Always zero on 'V'FAT HPFS NTFS*/
	short	st_align_for_word32; 	/* Align32 */
	unsigned short	st_mode; /* See above constants */
	short	st_nlink;	/* No. links 1 for non-NTFS msdocs say int */
	short	st_uid;		/*  good on NT? */
	short   st_gid; 	/*  good on NT? */
	short	st_align_for_word322; /* Align32 the Second */
	_dev_t	st_rdev;	/* set for dev handle */
	_off_t	st_size;	/* File size in bytes */
	time_t	st_atime;	/* Accessed date (always 00:00 hrs local) */
	time_t	st_ctime;	/* Creation time /these are backwards from MS*/
	time_t	st_mtime;	/* Modification time /docs, but right on VFAT */
};

#define __need___int64
#include <stddef.h>

struct _stati64
{
	_dev_t	st_dev;		/* device handle only con aux... otherwise 0 */
	_ino_t	st_ino;		/* Always zero on 'V'FAT HPFS NTFS*/
	short	st_align_for_word32; 	/* Align32 */
	unsigned short	st_mode; /* See above constants */
	short	st_nlink;	/* No. links 1 for non-NTFS msdocs say int */
	short	st_uid;		/*  good on NT? */
	short   st_gid; 	/*  good on NT? */
	short	st_align_for_word322; /* Align32 the Second */
	_dev_t	st_rdev;	/* set for dev handle */
	__int64	st_size;	/* File size in bytes */
	time_t	st_atime;	/* Accessed date (always 00:00 hrs local) */
	time_t	st_ctime;	/* Creation time /these are backwards from MS*/
	time_t	st_mtime;	/* Modification time /docs, but right on VFAT */
};

int	_fstat (int nHandle, struct _stat* pstat);
int     _fstati64( int handle, struct _stati64 *buffer );
int	_stat (const char* szPath, struct _stat* pstat);
__int64	_stati64 (const char* szPath, struct _stati64* pstat);

#ifdef _WIN32_WINNT
int _wstat( const wchar_t *path, struct _stat *buffer );
__int64 _wstati64( const wchar_t *path, struct _stati64 *buffer );
#endif //_WIN32_WINNT


#ifndef	_NO_OLDNAMES

/* These functions live in liboldnames.a. */
struct stat
{
	_dev_t	st_dev;		/* device handle only con aux... otherwise 0 */
	_ino_t	st_ino;		/* Always zero on 'V'FAT HPFS NTFS*/
	short	st_align_for_word32; 	/* Align32 */
	unsigned short	st_mode; /* See above constants */
	short	st_nlink;	/* No. links 1 for non-NTFS msdocs say int */
	short	st_uid;		/*  good on NT? */
	short   st_gid; 	/*  good on NT? */
	short	st_align_for_word322; /* Align32 the Second */
	_dev_t	st_rdev;	/* set for dev handle */
	_off_t	st_size;	/* File size in bytes */
	time_t	st_atime;	/* Accessed date (always 00:00 hrs local) */
	time_t	st_ctime;	/* Creation time /these are backwards from MS*/
	time_t	st_mtime;	/* Modification time /docs, but right on VFAT */
};

int	fstat (int nHandle, struct stat* pstat);
int	stat (const char* szPath, struct stat* pstat);

#endif	/* Not _NO_OLDNAMES */


#ifdef	__cplusplus
}
#endif

#endif	/* Not _STAT_H_ */

#endif	/* Not _STRICT_ANSI */
