/*
 * tchar.h
 *
 * Unicode mapping layer for the standard C library. By including this
 * file and using the 't' names for string functions
 * (eg. _tprintf) you can make code which can be easily adapted to both
 * Unicode and non-unicode environments. In a unicode enabled compile define
 * _UNICODE before including tchar.h, otherwise the standard non-unicode
 * library functions will be used.
 *
 * Note that you still need to include string.h or stdlib.h etc. to define
 * the appropriate functions. Also note that there are several defines
 * included for non-ANSI functions which are commonly available (but using
 * the convention of prepending an underscore to non-ANSI library function
 * names).
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Updated 97/09/11 by Mikey <jeffdb@netzone.com>
 *  for use with msvcrt20.dll && _MBCS
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _TCHAR_H_
#define _TCHAR_H_


#if !defined(_MBCS) && !defined(_UNICODE)

/*
 * _TCHAR, the type you should use instead of char.
 */

typedef char    _TCHAR;
typedef int     _TINT;
typedef signed char _TSCHAR;
typedef unsigned char _TUCHAR;


/*
 * Enclose constant strings and characters in the _TEXT macro.
 */

#define _TEXT(x)        x
#define __T(x)		x
#define _tenviron	_environ
#define _tfinddata_t	_finddata_t


/*
 * Non-unicode (standard) functions
 */


#define _fgettc      fgetc
#define _fgettchar   fgetchar
#define _fgetts      fgets
#define _fputtc      fputc
#define _fputtchar   fputchar
#define _fputts      fputs
#define _ftprintf    fprintf
#define _ftscanf     fscanf
#define _gettc       getc
#define _gettchar    getchar
#define _getts       gets
#define _istalnum    isalnum
#define _istalnum    isalnum
#define _istalpha    isalpha
#define _istascii    __isascii
#define _istcntrl    iscntrl
#define _istdigit    isdigit
#define _istgraph    isgraph
#define _istlead     FALSE
#define _istleadbyte FALSE
#define _istlegal    TRUE
#define _istlower    islower
#define _istprint    isprint
#define _istpunct    ispunct
#define _istspace    isspace
#define _istupper    isupper
#define _istxdigit   isxdigit
#define _itot        _itoa
#define _ltot        _ltoa
#define _puttc       putc
#define _puttchar    putchar
#define _putts       puts
#define _tmain       main
#define _sntprintf   _snprintf
#define _stprintf    sprintf
#define _stscanf     sscanf
#define _stscanf     sscanf
#define _taccess     _access
#define _tasctime    asctime
#define _tccpy       Maps_to
#define _tchdir      _chdir
#define _tclen       Maps_to
#define _tchmod      _chmod
#define _tcreat      _creat
#define _tcscat      strcat
#define _tcschr      strchr
#define _tcsclen     strlen
#define _tcscmp      strcmp
#define _tcscoll     strcoll
#define _tcscpy      strcpy
#define _tcscspn     strcspn
#define _tcsdec      _strdec
#define _tcsdup      _strdup
#define _tcsftime    strftime
#define _tcsicmp     _stricmp
#define _tcsicoll    _stricoll
#define _tcsinc      _strinc
#define _tcslen      strlen
#define _tcslwr      _strlwr
#define _tcsnbcnt    _strncnt
#define _tcsnbcnt    _strncnt
#define _tcsncat     strncat
#define _tcsnccat    strncat
#define _tcsncmp     strncmp
#define _tcsnccmp    strncmp
#define _tcsnccnt    _strncnt
#define _tcsnccpy    strncpy
#define _tcsncicmp   _strnicmp
#define _tcsncpy     strncpy
#define _tcsncset    _strnset
#define _tcsnextc    _strnextc
#define _tcsnicmp    _strnicmp
#define _tcsnicoll   _strnicoll
#define _tcsninc     _strninc
#define _tcsnccnt    _strncnt
#define _tcsnset     _strnset
#define _tcspbrk     strpbrk
#define _tcsspnp     _strspnp
#define _tcsrchr     strrchr
#define _tcsrev      _strrev
#define _tcsset      _strset
#define _tcsspn      strspn
#define _tcsstr      strstr
#define _tcstod      strtod
#define _tcstok      strtok
#define _tcstol      strtol
#define _tcstoul     strtoul
#define _tcstoul     strtoul
#define _tcsupr      _strupr
#define _tcsxfrm     strxfrm
#define _tctime      ctime
#define _texecl      _execl
#define _texecle     _execle
#define _texeclp     _execlp
#define _texeclpe    _execlpe
#define _texecv      _execv
#define _texecve     _execve
#define _texecvp     _execvp
#define _texecvpe    _execvpe
#define _tfdopen     _fdopen
#define _tfindfirst  _findfirst
#define _tfindnext   _findnext
#define _tfopen      fopen
#define _tfreopen    freopen
#define _tfsopen     _fsopen
#define _tfullpath   _fullpath
#define _tgetcwd     _getcwd
#define _tgetenv     getenv
#define _tmain       main
#define _tmakepath   _makepath
#define _tmkdir      _mkdir
#define _tmktemp     _mktemp
#define _tperror     perror
#define _topen       _open
#define _topen       _open
#define _totlower    tolower
#define _totupper    toupper
#define _tpopen      _popen
#define _tprintf     printf
#define _tremove     remove
#define _trename     rename
#define _trmdir      _rmdir
#define _tsearchenv  _searchenv
#define _tscanf      scanf
#define _tsetlocale  setlocale
#define _tsopen      _sopen
#define _tspawnl     _spawnl
#define _tspawnle    _spawnle
#define _tspawnlp    _spawnlp
#define _tspawnlpe   _spawnlpe
#define _tspawnv     _spawnv
#define _tspawnve    _spawnve
#define _tspawnvp    _spawnvp
#define _tspawnvpe   _spawnvpe
#define _tsplitpath  _splitpath
#define _tstat       _stat
#define _tstrdate    _strdate
#define _tstrtime    _strtime
#define _tsystem     system
#define _ttempnam    _tempnam
#define _ttmpnam     tmpnam
#define _ttmpnam     tmpnam
#define _ttoi        atoi
#define _ttol        atol
#define _tutime      _utime
#define _tWinMain    WinMain
#define _ultot       _ultoa
#define _ungettc     ungetc
#define _vftprintf   vfprintf
#define _vsntprintf  _vsnprintf
#define _vstprintf   vsprintf
#define _vtprintf    vprintf

#endif // !_MBCS && !_UNICODE
#if defined(_MBCS)

/*
 * TCHAR, the type you should use instead of char.
 */

typedef char    _TCHAR;
typedef int	_TINT;
typedef signed char _TSCHAR;
typedef unsigned char _TUCHAR;


/*
 * Enclose constant strings and characters in the _TEXT macro.
 */
#define _TEXT(x)        x
#define __T(x)		x
#define _tenviron       _environ
#define _tfinddata_t    _finddata_t


/*
 * Non-unicode (multi-byte) functions
 */


#define _fgettc		fgetc        
#define _fgettchar	fgetchar     
#define _fgetts		fgets        
#define _fputtc		fputc        
#define _fputtchar	fputchar     
#define _fputts		fputs        
#define _ftprintf	fprintf      
#define _ftscanf	fscanf       
#define _gettc		getc         
#define _gettchar	getchar      
#define _getts		gets         
#define _istalnum	_ismbcalnum  
#define _istalnum	_ismbcalnum  
#define _istalpha	_ismbcalpha  
#define _istascii	__isascii    
#define _istcntrl	iscntrl      
#define _istdigit	_ismbcdigit  
#define _istgraph	_ismbcgraph  
#define _istlead	_ismbblead   
#define _istleadbyte	isleadbyte   
#define _istlegal	_ismbclegal  
#define _istlower	_ismbclower  
#define _istprint	_ismbcprint  
#define _istpunct	_ismbcpunct  
#define _istspace	_ismbcspace  
#define _istupper	_ismbcupper  
#define _istxdigit	isxdigit     
#define _itot		_itoa        
#define _ltot		_ltoa        
#define _puttc		putc         
#define _puttchar	putchar      
#define _putts		puts         
#define _tmain		main         
#define _sntprintf	_snprintf    
#define _stprintf	sprintf      
#define _stscanf	sscanf       
#define _stscanf	sscanf       
#define _taccess	_access      
#define _tasctime	asctime      
#define _tccpy		_mbccpy      
#define _tchdir		_chdir       
#define _tclen		_mbclen      
#define _tchmod		_chmod       
#define _tcreat		_creat       
#define _tcscat		_mbscat      
#define _tcschr		_mbschr      
#define _tcsclen	_mbslen      
#define _tcscmp		_mbscmp      
#define _tcscoll	_mbscoll     
#define _tcscpy		_mbscpy      
#define _tcscspn	_mbscspn     
#define _tcsdec		_mbsdec      
#define _tcsdup		_mbsdup      
#define _tcsftime	strftime     
#define _tcsicmp	_mbsicmp     
#define _tcsicoll	_stricoll    
#define _tcsinc		_mbsinc      
#define _tcslen		strlen       
#define _tcslwr		_mbslwr      
#define _tcsnbcnt	_mbsnbcnt    
#define _tcsnbcnt	_mbsnbcnt    
#define _tcsncat	_mbsnbcat    
#define _tcsnccat	_mbsncat     
#define _tcsncmp	_mbsnbcmp    
#define _tcsnccmp	_mbsncmp     
#define _tcsnccnt	_mbsnccnt    
#define _tcsnccpy	_mbsncpy     
#define _tcsncicmp	_mbsnicmp    
#define _tcsncpy	_mbsnbcpy    
#define _tcsncset	_mbsnset     
#define _tcsnextc	_mbsnextc    
#define _tcsnicmp	_mbsnicmp    
#define _tcsnicoll	_strnicoll   
#define _tcsninc	_mbsninc     
#define _tcsnccnt	_mbsnccnt    
#define _tcsnset	_mbsnbset    
#define _tcspbrk	_mbspbrk     
#define _tcsspnp	_mbsspnp     
#define _tcsrchr	_mbsrchr     
#define _tcsrev		_mbsrev      
#define _tcsset		_mbsset      
#define _tcsspn		_mbsspn      
#define _tcsstr		_mbsstr      
#define _tcstod		strtod       
#define _tcstok		_mbstok      
#define _tcstol		strtol       
#define _tcstoul	strtoul      
#define _tcstoul	strtoul      
#define _tcsupr		_mbsupr      
#define _tcsxfrm	strxfrm      
#define _tctime		ctime        
#define _texecl		_execl       
#define _texecle	_execle      
#define _texeclp	_execlp      
#define _texeclpe	_execlpe     
#define _texecv		_execv       
#define _texecve	_execve      
#define _texecvp	_execvp      
#define _texecvpe	_execvpe     
#define _tfdopen	_fdopen      
#define _tfindfirst	_findfirst   
#define _tfindnext	_findnext    
#define _tfopen		fopen        
#define _tfreopen	freopen      
#define _tfsopen	_fsopen      
#define _tfullpath	_fullpath    
#define _tgetcwd	_getcwd      
#define _tgetenv	getenv       
#define _tmain		main         
#define _tmakepath	_makepath    
#define _tmkdir		_mkdir       
#define _tmktemp	_mktemp      
#define _tperror	perror       
#define _topen		_open        
#define _topen		_open        
#define _totlower	_mbctolower  
#define _totupper	_mbctoupper  
#define _tpopen		_popen       
#define _tprintf	printf       
#define _tremove	remove       
#define _trename	rename       
#define _trmdir		_rmdir       
#define _tsearchenv	_searchenv   
#define _tscanf		scanf        
#define _tsetlocale	setlocale    
#define _tsopen		_sopen       
#define _tspawnl	_spawnl      
#define _tspawnle	_spawnle     
#define _tspawnlp	_spawnlp     
#define _tspawnlpe	_spawnlpe    
#define _tspawnv	_spawnv      
#define _tspawnve	_spawnve     
#define _tspawnvp	_spawnvp     
#define _tspawnvpe	_spawnvpe    
#define _tsplitpath	_splitpath   
#define _tstat		_stat        
#define _tstrdate	_strdate     
#define _tstrtime	_strtime     
#define _tsystem	system       
#define _ttempnam	_tempnam     
#define _ttmpnam	tmpnam       
#define _ttmpnam	tmpnam       
#define _ttoi		atoi         
#define _ttol		atol         
#define _tutime		_utime       
#define _tWinMain	WinMain      
#define _ultot		_ultoa       
#define _ungettc	ungetc       
#define _vftprintf	vfprintf     
#define _vsntprintf	_vsnprintf   
#define _vstprintf	vsprintf     
#define _vtprintf	vprintf      

#endif // !_MBCS
#if defined(_UNICODE)

#define __need_wchar_t
#define __need_wint_t
#include <stddef.h>

/*
 * _TCHAR, the type you should use instead of char.
 */
typedef wchar_t    _TCHAR;
typedef wint_t     _TINT;
typedef wchar_t _TSCHAR;
typedef wchar_t _TUCHAR;

/*
 * Enclose constant strings and characters in the _TEXT macro.
 */

#define _TEXT(x)	L ##   x
#define __T(x)		L ##   x
#define _TEOF		WEOF
#define _tenviron	_wenviron
#define _finddata_t	_wfinddata_t

/*
 * Unicode (international) functions
 */


#define _fgettc		fgetwc
#define _fgettchar	_fgetwchar
#define _fgetts		fgetws
#define _fputtc		fputwc
#define _fputtchar	_fputwchar
#define _fputts		fputws
#define _ftprintf	fwprintf
#define _ftscanf	fwscanf
#define _gettc		getwc
#define _gettchar	getwchar
#define _getts		getws
#define _istalnum	iswalnum
#define _istalnum	iswalnum
#define _istalpha	iswalpha
#define _istascii	iswascii
#define _istcntrl	iswcntrl
#define _istdigit	iswdigit
#define _istgraph	iswgraph
#define _istlead	FALSE
#define _istleadbyte	FALSE
#define _istlegal	TRUE
#define _istlower	iswlower
#define _istprint	iswprint
#define _istpunct	iswpunct
#define _istspace	iswspace
#define _istupper	iswupper
#define _istxdigit	iswxdigit
#define _itot		_itow
#define _ltot		_ltow
#define _puttc		putwc
#define _puttchar	putwchar
#define _putts		putws
#define _tmain		wmain
#define _sntprintf	_snwprintf
#define _stprintf	swprintf
#define _stscanf	swscanf
#define _stscanf	swscanf
#define _taccess	_waccess
#define _tasctime	_wasctime
#define _tccpy		_wccpy // FIXME
#define _tchdir		_wchdir
#define _tclen		_wclen // FIXME
#define _tchmod		_wchmod
#define _tcreat		_wcreat
#define _tcscat		wcscat
#define _tcschr		wcschr
#define _tcsclen	wcslen
#define _tcscmp		wcscmp
#define _tcscoll	wcscoll
#define _tcscpy		wcscpy
#define _tcscspn	wcscspn
#define _tcsdec		_wcsdec
#define _tcsdup		_wcsdup
#define _tcsftime	wcsftime
#define _tcsicmp	_wcsicmp
#define _tcsicoll	_wcsicoll
#define _tcsinc		_wcsinc
#define _tcslen		wcslen
#define _tcslwr		_wcslwr
#define _tcsnbcnt	_wcnscnt
#define _tcsnbcnt	_wcnscnt
#define _tcsncat	wcsncat
#define _tcsnccat	wcsncat
#define _tcsncmp	wcsncmp
#define _tcsnccmp	wcsncmp
#define _tcsnccnt	_wcsncnt
#define _tcsnccpy	wcsncpy
#define _tcsncicmp	_wcsnicmp
#define _tcsncpy	wcsncpy
#define _tcsncset	_wcsnset
#define _tcsnextc	_wcsnextc
#define _tcsnicmp	_wcsnicmp
#define _tcsnicoll	_wcsnicoll
#define _tcsninc	_wcsninc
#define _tcsnccnt	_wcsncnt
#define _tcsnset	_wcsnset
#define _tcspbrk	wcspbrk
#define _tcsspnp	_wcsspnp
#define _tcsrchr	wcsrchr
#define _tcsrev		_wcsrev
#define _tcsset		_wcsset
#define _tcsspn		wcsspn
#define _tcsstr		wcsstr
#define _tcstod		wcstod
#define _tcstok		wcstok
#define _tcstol		wcstol
#define _tcstoul	wcstoul
#define _tcstoul	wcstoul
#define _tcsupr		_wcsupr
#define _tcsxfrm	wcsxfrm
#define _tctime		_wctime
#define _texecl		_wexecl
#define _texecle	_wexecle
#define _texeclp	_wexeclp
#define _texeclpe	_wexeclpe
#define _texecv		_wexecv
#define _texecve	_wexecve
#define _texecvp	_wexecvp
#define _texecvpe	_wexecvpe
#define _tfdopen	_wfdopen
#define _tfindfirst	_wfindfirst
#define _tfindnext	_wfindnext
#define _tfopen		_wfopen
#define _tfreopen	_wfreopen
#define _tfsopen	_wfsopen
#define _tfullpath	_wfullpath
#define _tgetcwd	_wgetcwd
#define _tgetenv	_wgetenv
#define _tmain		wmain
#define _tmakepath	_wmakepath
#define _tmkdir		_wmkdir
#define _tmktemp	_wmktemp
#define _tperror	_wperror
#define _topen		_wopen
#define _topen		_wopen
#define _totlower	towlower
#define _totupper	towupper
#define _tpopen		_wpopen
#define _tprintf	wprintf
#define _tremove	_wremove
#define _trename	_wrename
#define _trmdir		_wrmdir
#define _tsearchenv	_wsearchenv
#define _tscanf		wscanf
#define _tsetlocale	_wsetlocale
#define _tsopen		_wsopen
#define _tspawnl	_wspawnl
#define _tspawnle	_wspawnle
#define _tspawnlp	_wspawnlp
#define _tspawnlpe	_wspawnlpe
#define _tspawnv	_wspawnv
#define _tspawnve	_wspawnve
#define _tspawnvp	_tspawnvp
#define _tspawnvpe	_tspawnvpe
#define _tsplitpath	_wsplitpath
#define _tstat		_wstat
#define _tstrdate	_wstrdate
#define _tstrtime	_wstrtime
#define _tsystem	_wsystem
#define _ttempnam	_wtempnam
#define _ttmpnam	_wtmpnam
#define _ttmpnam	_wtmpnam
#define _ttoi		_wtoi
#define _ttol		_wtol
#define _tutime		_wutime
#define _tWinMain	wWinMain
#define _ultot		_ultow
#define _ungettc	ungetwc
#define _vftprintf	vfwprintf
#define _vsntprintf	_vsnwprintf
#define _vstprintf	vswprintf
#define _vtprintf	vwprintf

#endif // _UNICODE

#endif /* _TCHAR_H_ not defined */
